package jumpstart.min.business.domain.security;

import java.util.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;

import jumpstart.min.business.commons.exception.AuthenticationException;
import jumpstart.min.business.commons.exception.DoesNotExistException;
import jumpstart.min.business.commons.query.ComparisonOperator;
import jumpstart.min.business.commons.query.QueryBuilder;
import jumpstart.min.business.commons.query.SearchOptions;
import jumpstart.min.business.domain.base.BaseSvc;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcRemote;
import jumpstart.min.business.domain.security.iface.UserSearchFields;

// @SecurityDomain("jumpstart")
@Stateless
@Local(ISecurityFinderSvcLocal.class)
@Remote(ISecurityFinderSvcRemote.class)
public class SecurityFinderSvc extends BaseSvc implements ISecurityFinderSvcLocal, ISecurityFinderSvcRemote {

	public User findUser(Long id) throws DoesNotExistException {
		User obj = (User) find(User.class, id);
		return obj;
	}

	public User authenticateUser(String loginId, String password) throws AuthenticationException {
		User user;
		try {
			user = findUserByLoginId(loginId);
		}
		catch (DoesNotExistException e) {
			throw new AuthenticationException("User_loginId_unknown", loginId);
		}
		user.authenticate(password);
		return user;
	}

	public User findUserByLoginId(String loginId) throws DoesNotExistException {

		try {
			Query q = _em.createQuery("select u from User u where u.loginId = :loginId");
			q.setParameter("loginId", loginId);
			User obj = (User) q.getSingleResult();
			return obj;
		}
		catch (NoResultException e) {
			throw new DoesNotExistException(User.class, loginId);
		}
		catch (NonUniqueResultException e) {
			throw new IllegalStateException("Duplicate User found with loginId = " + loginId + ".");
		}
	}

	@SuppressWarnings("unchecked")
	public List<User> findUsers() {

		Query q = _em.createQuery("select u from User u order by u.loginId");
		List l = q.getResultList();

		return l;
	}

	@SuppressWarnings("unchecked")
	public List<User> findUsers(UserSearchFields search, SearchOptions options) {

		QueryBuilder builder = new QueryBuilder("");
		builder.append("select u from User u");
		builder.appendEquals("u.active", search.getActive(), true);
		builder.appendLikeIgnoreCase("u.loginId", search.getLoginId());
		builder.appendLikeIgnoreCase("u.password", search.getPassword());
		builder.appendEquals("u.salutation", search.getSalutation(), true);
		builder.appendLikeIgnoreCase("u.firstName", search.getFirstName());
		builder.appendLikeIgnoreCase("u.lastName", search.getLastName());
		builder.appendLikeIgnoreCase("u.emailAddress", search.getEmailAddress());
		builder.appendComparison("u.expiryDate", ComparisonOperator.EQ, search.getExpiryDate());
		builder.appendEquals("u.version", search.getVersion(), true);

		if (options.getSortColumnNames().size() == 0) {
			builder.append(" order by u.loginId");
		}

		Query q = builder.createQuery(_em, options, "u");

		List l = q.getResultList();

		return l;
	}

}
