package jumpstart.min.web.components;

import jumpstart.min.web.pages.login.AboutPage;
import jumpstart.min.web.pages.login.HelpPage;
import jumpstart.min.web.pages.security.UserSearchPage;
import jumpstart.min.web.state.Visit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.BaseComponent;
import org.apache.tapestry.IAsset;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.annotations.Asset;
import org.apache.tapestry.annotations.InjectObject;
import org.apache.tapestry.annotations.InjectState;
import org.apache.tapestry.annotations.Parameter;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.event.PageBeginRenderListener;
import org.apache.tapestry.event.PageEvent;

/**
 * Common navigational border for jumpstart.
 * 
 * @author Geoff Callender
 */

public abstract class Border extends BaseComponent implements PageBeginRenderListener {
	static private Log LOG = LogFactory.getLog(Border.class);

	//
	// INJECTION BUG WARNING...
	// If you suffer from the sporadic Tapestry 4.0 bug where injection
	// sometimes fails in components when annotations and
	// -Dorg.apache.tapestry.disable-caching=true are used together
	// then comment out the parameters and assets below
	// and uncomment their counterparts in Border.jwc
	//

	@Parameter(name = "tab", required = false, defaultValue = "")
	public abstract String getActiveTab();

	@Asset("context:/css/global.css")
	public abstract IAsset getStylesheet();

	@InjectState("visit")
	public abstract Visit getMyVisit();

	@InjectObject("engine-service:restart")
	public abstract IEngineService getRestartService();

	public void pageBeginRender(PageEvent event) {
	}

	//
	// Listeners for links at top of page
	//

	public void doHelp(IRequestCycle cycle) {
		HelpPage nextPage = (HelpPage) cycle.getPage(HelpPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doAbout(IRequestCycle cycle) {
		AboutPage nextPage = (AboutPage) cycle.getPage(AboutPage.PAGE_NAME);
		nextPage.activate();
	}

	public ILink doLogout(IRequestCycle cycle) {
		LOG.info(getMyVisit().getMyLoginId() + " is logging out.");
		
		// Redirect to the restart service, which will destroy the session and
		// therefore cleans up our Visit

		ILink redirectTo = getRestartService().getLink(false, null);
		return redirectTo;
	}

	//
	// Listeners for tabs
	//

	public void doSecurityTab(IRequestCycle cycle) {
		UserSearchPage nextPage = (UserSearchPage) cycle.getPage(UserSearchPage.PAGE_NAME);
		nextPage.activate();
	}

}
