package jumpstart.min.web.pages.login;

import jumpstart.min.business.commons.exception.BusinessException;
import jumpstart.min.business.domain.security.User;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.min.web.pages.base.SimpleBasePage;
import jumpstart.min.web.pages.examples.beanform.BeanFormPage;
import jumpstart.min.web.pages.examples.infrastructure.EngineServicesAndRestartPage;
import jumpstart.min.web.pages.examples.simple.SimpleEditPage;
import jumpstart.min.web.pages.examples.simple.SimpleInputPage;
import jumpstart.min.web.pages.examples.simple.SimpleOutputUsingActivatePage;
import jumpstart.min.web.pages.examples.simple.SimpleValidationPage;
import jumpstart.min.web.pages.examples.styling.CSSPage;
import jumpstart.min.web.pages.examples.tables.AlternatingTablePage;
import jumpstart.min.web.pages.examples.tables.PreviewableTablePage;
import jumpstart.min.web.pages.examples.tables.TablePage;
import jumpstart.min.web.pages.examples.tables.TableWithLinksPage;
import jumpstart.min.web.pages.examples.validators.ClientValidatorsPage;
import jumpstart.min.web.pages.examples.validators.MixedValidationPage;
import jumpstart.min.web.pages.examples.validators.ValidatorsPage;
import jumpstart.min.web.state.Visit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.annotations.InjectObject;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;

/**
 * Allows the user to login, by providing userName and password. After succesfully logging in.
 * 
 * @author Geoff Callender
 */

public abstract class LoginPage extends SimpleBasePage {
	static public final Log LOG = LogFactory.getLog(LoginPage.class);

	static public final String PAGE_NAME = "pages/login/LoginPage";

	@InjectObject("engine-service:page")
	public abstract IEngineService getPageService();

	// @Asset("context:/css/login.css")
	// In Tapestry 4.1, have to use <asset> in LoginPage.page instead of @Asset
	// here. Otherwise a problem occurs when when other pages activate this
	// page: "Component pages/login/LoginPage does not contain an asset named
	// 'stylesheet'."
	// Wasn't a problem in Tapestry 4.0. Might only occur when there's a .page.
	// public abstract IAsset getStylesheet();

	public abstract void setUserName(String value);

	public abstract String getLoginId();

	public abstract void setPassword(String value);
	public abstract String getPassword();

	public void activate() {
		// No preparation required before activation.
		getRequestCycle().activate(this);
	}

	public void doToSimpleOutputUsingActivate() {
		SimpleOutputUsingActivatePage nextPage = (SimpleOutputUsingActivatePage) getRequestCycle().getPage(
				SimpleOutputUsingActivatePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToSimpleInput() {
		SimpleInputPage nextPage = (SimpleInputPage) getRequestCycle().getPage(SimpleInputPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToSimpleValidation() {
		SimpleValidationPage nextPage = (SimpleValidationPage) getRequestCycle()
				.getPage(SimpleValidationPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToSimpleEdit() {
		SimpleEditPage nextPage = (SimpleEditPage) getRequestCycle().getPage(SimpleEditPage.PAGE_NAME);
		Long personId = new Long(1);
		nextPage.activate(personId);
	}

	public void doToValidators() {
		ValidatorsPage nextPage = (ValidatorsPage) getRequestCycle().getPage(ValidatorsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToClientValidators() {
		ClientValidatorsPage nextPage = (ClientValidatorsPage) getRequestCycle()
				.getPage(ClientValidatorsPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToMixedValidation() {
		MixedValidationPage nextPage = (MixedValidationPage) getRequestCycle().getPage(MixedValidationPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToBeanForm() {
		BeanFormPage nextPage = (BeanFormPage) getRequestCycle().getPage(BeanFormPage.PAGE_NAME);
		Long personId = new Long(1);
		nextPage.activate(personId);
	}

	public void doToCSS() {
		CSSPage nextPage = (CSSPage) getRequestCycle().getPage(CSSPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTable() {
		TablePage nextPage = (TablePage) getRequestCycle().getPage(TablePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToTableWithLinks() {
		TableWithLinksPage nextPage = (TableWithLinksPage) getRequestCycle().getPage(TableWithLinksPage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToPreviewableTable() {
		PreviewableTablePage nextPage = (PreviewableTablePage) getRequestCycle()
				.getPage(PreviewableTablePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doToAlternatingTable() {
		AlternatingTablePage nextPage = (AlternatingTablePage) getRequestCycle()
				.getPage(AlternatingTablePage.PAGE_NAME);
		nextPage.activate();
	}

	public void doEngineServicesAndRestart() {
		EngineServicesAndRestartPage nextPage = (EngineServicesAndRestartPage) getRequestCycle().getPage(
				EngineServicesAndRestartPage.PAGE_NAME);
		nextPage.activate();
	}

	public ILink doLogin() {
		ILink redirectTo = null;

		try {
			validate();

			if (hasErrors()) {
				return null;
			}

			// Authenticate the user, password

			ISecurityFinderSvcLocal finder = getBusinessServicesLocator().getSecurityFinderSvcLocal();
			User user = finder.authenticateUser(getLoginId(), getPassword());

			// Store the user and roles in the Visit

			Visit visit = getMyVisit();
			visit.noteLogIn(user);
			LOG.info(user.getLoginId() + " has logged in.");

			// Redirect to next page.
			// SECURITY NOTE - do not use simple page activation Instead, we
			// return an ILink because that instructs Tapestry to do
			// a redirect-after-post.
			// Redirect-after-post protects against a security vulnerability in
			// which a user may log in without a password simply by using a
			// browser that has been logged out, then using the Back button to
			// go back to the first screen after the login screen and
			// pressing refresh.
			// Without redirect-after-post this would re-post the login info
			// and they'd be logged in!

			redirectTo = getPageService().getLink(false, WelcomePage.PAGE_NAME);
		}
		catch (BusinessException e) {
			recordError(e.toString());
		}
		catch (Exception e) {
			e.printStackTrace();
			recordError(getMessages().getMessage("login_problem"));
		}

		return redirectTo;
	}

	private void validate() {
		// None to do - validation has been specified in the HTML template
	}

	public void validate(IRequestCycle cycle) {
		super.validate(cycle);
	}

}
