package jumpstart.min.web.pages.security;

import jumpstart.min.business.commons.exception.BusinessException;
import jumpstart.min.business.domain.security.User;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.min.web.pages.base.ProtectedBasePage;

import org.apache.tapestry.annotations.Persist;
import org.apache.tapestry.form.IPropertySelectionModel;
import org.apache.tapestry.form.StringPropertySelectionModel;

public abstract class UserEditPage extends ProtectedBasePage {
	// static private final Log LOG = LogFactory.getLog(UserEditPage.class);
	static public final String PAGE_NAME = "pages/security/UserEditPage";

	@Persist("client")
	public abstract Long getUserId();
	public abstract void setUserId(Long value);

	@Persist("session")
	// Probably best not to use client persistence, although it does encode the
	// data, because User inludes password
	public abstract User getUser();
	public abstract void setUser(User value);

	@Persist("session")
	// Cannot use client persistence because not serializable
	public abstract IPropertySelectionModel getSalutationsModel();
	public abstract void setSalutationsModel(IPropertySelectionModel value);

	public void activate(Long id) throws BusinessException {
		setUserId(id);

		// Get the User

		ISecurityFinderSvcLocal securityFinder = getBusinessServicesLocator().getSecurityFinderSvcLocal();
		User user = securityFinder.findUser(id);
		setUser(user);

		// Put a selection list of SALUTATIONS onto the page

		setSalutationsModel(new StringPropertySelectionModel(User.SALUTATIONS));

		getRequestCycle().activate(this);
	}

	public void doSave() {

		if (!hasErrors()) {
			validateSave();

			if (!hasErrors()) {
				try {
					ISecurityManagerSvcLocal manager = getBusinessServicesLocator().getSecurityManagerSvcLocal();

					// Change the User

					User user = getUser();
					manager.changeUser(user);

					UserSearchPage nextPage = (UserSearchPage) getRequestCycle().getPage(UserSearchPage.PAGE_NAME);
					nextPage.activateAgain();
				}
				catch (Exception e) {
					handleBusinessServicesExceptionForChange(e);
				}
			}
		}

	}

	private void validateSave() {
		// if (xyz) {
		// recordError(getMessages().getMessage("Xyz_error"));
		// }
	}

	public void doChangePassword() {

		try {
			Long id = getUserId();
			UserPasswordChangePage nextPage = (UserPasswordChangePage) getRequestCycle().getPage(
					UserPasswordChangePage.PAGE_NAME);
			nextPage.activate(id);
		}
		catch (BusinessException e) {
			recordError(e.toString());
		}

	}

	public void doCancel() {
		UserSearchPage nextPage = (UserSearchPage) getRequestCycle().getPage(UserSearchPage.PAGE_NAME);
		nextPage.activateAgain();
	}

}
