package jumpstart.min.web.pages.security;

import jumpstart.min.business.commons.exception.BusinessException;
import jumpstart.min.business.domain.security.User;
import jumpstart.min.business.domain.security.iface.ISecurityFinderSvcLocal;
import jumpstart.min.business.domain.security.iface.ISecurityManagerSvcLocal;
import jumpstart.min.web.pages.base.ProtectedBasePage;

import org.apache.tapestry.annotations.Persist;

public abstract class UserPasswordChangePage extends ProtectedBasePage {
	static public final String PAGE_NAME = "pages/security/UserPasswordChangePage";

	@Persist("client")
	public abstract Long getUserId();
	public abstract void setUserId(Long value);

	@Persist("session")
	// Probably best not to use client persistence, although it does encode the
	// data, because User inludes password
	public abstract User getUser();
	public abstract void setUser(User value);

	public abstract String getNewPassword();
	public abstract String getConfirmNewPassword();

	public void activate(Long id) throws BusinessException {
		setUserId(id);

		// Get the User

		ISecurityFinderSvcLocal securityFinder = getBusinessServicesLocator().getSecurityFinderSvcLocal();
		User user = securityFinder.findUser(id);
		setUser(user);

		getRequestCycle().activate(this);
	}

	public void doSave() {

		if (!hasErrors()) {
			validateSave();

			if (!hasErrors()) {
				try {
					ISecurityManagerSvcLocal manager = getBusinessServicesLocator().getSecurityManagerSvcLocal();

					// Change the User password

					manager.changeUserPassword(getUserId(), getNewPassword());

					UserEditPage nextPage = (UserEditPage) getRequestCycle().getPage(UserEditPage.PAGE_NAME);
					nextPage.activate(getUserId());
				}
				catch (Exception e) {
					handleBusinessServicesExceptionForChange(e);
				}
			}
		}

	}

	private void validateSave() {

		if (!getNewPassword().equals(getConfirmNewPassword())) {
			recordError(getMessages().getMessage("User_confirmnewpassword_does_not_match"), "newPassword");
		}
	}

	public void doCancel() {
		try {
			UserEditPage nextPage = (UserEditPage) getRequestCycle().getPage(UserEditPage.PAGE_NAME);
			nextPage.activate(getUserId());
		}
		catch (BusinessException e) {
			recordError(e.toString());
		}
	}

}
